import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/*****
 *
 *  Provides the View for a Simple Web Crawler
 *  keep in mind it is called a web crawler because it can be very slow!!
 */
public class View extends JFrame
{
	public static final int FRAMEHEIGHT = 800;
	public static final int FRAMEWIDTH = 1400;
	
	public static final int DISPLAYHEIGHT = 80;
	public static final int DISPLAYWIDTH = 35;

	private JLabel startURLLabel;
	private JLabel searchStringLabel;
	private JLabel searchLimitLabel;
	private JLabel resultsLabel;
	
	private JButton searchButton;
	private JButton clearButton;
	
	
	private JTextField startURLField;
	private JTextField searchStringField;
	private JTextField searchLimitField;
	
	private JScrollPane pane;
	private JTextArea displayArea;
	
	private JPanel inputPanel;
	private JPanel startPanel;
	private JPanel searchPanel;
	private JPanel limitPanel;
	private JPanel buttonPanel;
	private JPanel outputPanel;
	
	
	/**
	 * Constructor for the View class loads and displays the frame
	 */
	public View(){
		
		//set up the display area
		displayArea = new JTextArea(DISPLAYHEIGHT, DISPLAYWIDTH);
		pane = new JScrollPane(displayArea);
		pane.setPreferredSize(new Dimension(450, FRAMEHEIGHT - 50));
		pane.revalidate();
		
		//call private helper methods to set up buttons and fields
		setUpButtons();
		setUpStart();
		setUpSearch();
		setUpLimit();
		
		//set up title panel
		JPanel TitlePanel = new JPanel();
		TitlePanel.setLayout(new BorderLayout());
		JLabel title = new JLabel("Web Crawler Demo");
		title.setHorizontalAlignment(JLabel.CENTER);
		TitlePanel.add(title, BorderLayout.CENTER);
		
		//set up input panel
		inputPanel = new JPanel();
		inputPanel.setLayout(new GridLayout(8,1));
		inputPanel.add(TitlePanel);
		inputPanel.add(startPanel);
		inputPanel.add(searchPanel);
		inputPanel.add(limitPanel);
		inputPanel.add(buttonPanel);
		
		//set up output panel
		outputPanel = new JPanel();
		outputPanel.add(pane);
		
		//add panels to frame
		this.setLayout(new BorderLayout());
		this.add(outputPanel, BorderLayout.EAST);
		this.add(inputPanel, BorderLayout.CENTER);
		
		//set the size and display the frame
		this.setSize(FRAMEWIDTH, FRAMEHEIGHT);
		this.setResizable(false);
		this.setVisible(true);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

	}
	
	/**
	 * private helper method to set up buttons
	 */
	private void setUpButtons()
	{
		searchButton = new JButton("search");
		searchButton.addActionListener(new searchListener());
		
		clearButton = new JButton("clear");
		clearButton.addActionListener(new clearListener());
		
		buttonPanel = new JPanel();
		buttonPanel.add(searchButton);
		buttonPanel.add(clearButton);
	}
	
	
	/**
	 * private helper method to set up start labels and fields
	 */
	private void setUpStart()
	{
		startURLLabel = new JLabel("Starting URL");
		startURLField = new JTextField(30);
		
		startPanel = new JPanel();
		SpringLayout layout = new SpringLayout();
		startPanel.setLayout(layout);
        startPanel.add(startURLLabel);
        startPanel.add(startURLField);
		
		layout.putConstraint(SpringLayout.WEST, startURLLabel,
							 5,
							 SpringLayout.WEST, startPanel);
		layout.putConstraint(SpringLayout.NORTH, startURLLabel,
							 5,
							 SpringLayout.NORTH, startPanel);
		
		layout.putConstraint(SpringLayout.WEST, startURLField,
							 5,
							 SpringLayout.EAST, startURLLabel);
		layout.putConstraint(SpringLayout.NORTH, startURLField,
							 5,
							 SpringLayout.NORTH, startPanel);
		
		
	}
	
	/**
	 * private helper method to set up search labels and fields
	 */
	private void setUpSearch()
	{
		searchStringLabel = new JLabel("Search String");
		
		searchStringField = new JTextField(30);
		
		searchPanel = new JPanel();
		SpringLayout layout2 = new SpringLayout();

		searchPanel.setLayout(layout2);
		searchPanel.add(searchStringLabel);
		searchPanel.add(searchStringField);
		
		layout2.putConstraint(SpringLayout.WEST, searchStringLabel,
							 5,
							 SpringLayout.WEST, searchPanel);
		layout2.putConstraint(SpringLayout.NORTH, searchStringLabel,
							 5,
							 SpringLayout.NORTH, searchPanel);
		
		layout2.putConstraint(SpringLayout.WEST, searchStringField,
							 5,
							 SpringLayout.EAST, searchStringLabel);
		layout2.putConstraint(SpringLayout.NORTH, searchStringField,
							 5,
							 SpringLayout.NORTH, searchPanel);
				
	}
	
	/**
	 * private helper method to set up limit labels and fields
	 */
	private void setUpLimit()
	{
		searchLimitLabel = new JLabel("Number of pages to search");
		
		searchLimitField = new JTextField(5);
		
		SpringLayout layout3 = new SpringLayout();

		limitPanel = new JPanel();
		limitPanel.setLayout(layout3);
		limitPanel.add(searchLimitLabel);
		limitPanel.add(searchLimitField);
		
		layout3.putConstraint(SpringLayout.WEST, searchLimitLabel,
							  5,
							  SpringLayout.WEST, limitPanel);
		layout3.putConstraint(SpringLayout.NORTH, searchLimitLabel,
							  5,
							  SpringLayout.NORTH, limitPanel);
		
		layout3.putConstraint(SpringLayout.WEST, searchLimitField,
							  5,
							  SpringLayout.EAST, searchLimitLabel);
		layout3.putConstraint(SpringLayout.NORTH, searchLimitField,
							  5,
							  SpringLayout.NORTH, limitPanel);
				
	}
	

	
	/**
	 * private inner class searchListener provides the action listener for the save button
	 */
	
	private class searchListener implements ActionListener{
		
		/**
		 * This method provides the action listener for the search button
		 *
		 * @param e the action event handled by this method
		 */
		
		public void actionPerformed(ActionEvent e)
		{ 
			try 
			{
				displayArea.setText("Searching . . .");
				displayArea.paintImmediately(0,0,FRAMEWIDTH, FRAMEHEIGHT);
			    WebSearch w = new WebSearch(startURLField.getText());
			    String searchString = searchStringField.getText();
			    int searchLimit = Integer.parseInt(searchLimitField.getText());
			    displayArea.setText(w.SearchURL(searchString, searchLimit));
			}
			catch (Exception ex) 
			{
				displayArea.setText(ex.toString());
				ex.printStackTrace();
			}
		}
	}
	
	/**
	 * private inner class clearListener provides the action listener for the clear button
	 */
	
	private class clearListener implements ActionListener
	{
		
		/**
		 * This method provides the action listener for the clear button
		 *
		 * @param e the action event handled by this method
		 */
		
		public void actionPerformed(ActionEvent e)
		{ 
			
			startURLField.setText("");
			searchStringField.setText("");
			searchLimitField.setText("");
			displayArea.setText("");
			
		}
	}
}
