
import java.net.*;
import java.io.*;
import java.util.*;
import org.apache.commons.io.IOUtils;
/*****
 *
 *  Provides the Search engine for a partial solution to a Simple Web Crawler
 *  keep in mind it is called a web crawler because it can be very slow!!!
 */

public class WebSearch {

	private URL startingURL;
	
	/**
	 * Constructor for the WebSearch creates a new URL object from the given String
	 * @param startingURL String representing the starting point of the search
	 * @throws Exception if the String is not a URL
	 */	
	public WebSearch(String startingURL) throws Exception
	{
		this.startingURL = new URL(startingURL);
	}
	
	/**
	 * SearchURL method for the WebSearch performs a breadth-first search starting at
	 * the given URL for the search string and continues until the search limit is reached
	 * @param searchString String representing the object of the search
	 * @param searchLimit provides the limit of how many searches to perform
	 */
	public String SearchURL(String searchString, int searchLimit) 
	{
		//set up queue to manage links and add the starting point
		//if we used a stack, it would be a depth-first search instead of breadth-first
		Queue<URL> queue = new LinkedList<URL>();
		queue.add(startingURL);
		
		boolean done = false;
		int counter = 0;
		int beginIndex;
		int endIndex;
		String result = "Displaying sites that contain the search string \n \n";
		String currentPage;
		URL currentURL;
		
		while ((!done) && (!queue.isEmpty()))
		{
			try 
			{
			    currentURL = queue.remove();
			    currentPage = readURL(currentURL);
			
				if (currentPage.indexOf(searchString) > 0)
			        result = result + currentURL.toString() + "\n";
			
			    beginIndex=0;
			    endIndex=0;
			
			    while (beginIndex > -1)
			    {
				    beginIndex = currentPage.substring(endIndex).indexOf("http://");
				    if (beginIndex > -1)
					{
					    beginIndex = beginIndex + endIndex;
				        endIndex = currentPage.substring(beginIndex).indexOf("\"") + beginIndex;
					    
				        String link = currentPage.substring(beginIndex, endIndex);
					    
					    if (endIndex>beginIndex)
					        queue.add(new URL(link));
				    }
			    }
			}
			catch (Exception ex)
			{
				//ignore exceptions and continue searching
			}
			
			
			counter++;
			
			done = counter > searchLimit;
		}
		
		if (result.equals(""))
			result = "No search results found";
	    return result;
	}
			
		
	/**
	 *
	 * private helper method demonstrates how to read the contents of a URL as a String.
	 *
	 * This version improves upon version 1 through the use of IOUtils 
	 *
	 * @param webservice provides the URL address of the web service to be accessed
	 * @return String representation of the given web page or service
	 * @throws java.net.MalformedURLException if the given url is poorly formed
	 * @throws java.io.IOException if IOUtils encounters an IOException
	 *
	 **/    
	private static String readURL(URL webservice) throws java.net.MalformedURLException, java.io.IOException
	{
		// use IOUtils to access the URL and return a string		
		String result = IOUtils.toString(webservice, "UTF-8");
		return result;
    }
}


